Attribute VB_Name = "modGetLastError1"
Option Explicit

'================================================================================
'
' modGetLastError1.bas
' --------------------
'
' Created By   : Kevin Wilson
'                http://w3.one.net/~kevinw/VB   (The VB Zone)
'
' Last Updated : July 20, 2000
'
' VB Versions  : 5.0 / 6.0
'
' Requires     : Nothing
'
' Description  : This module contains one function that is designed to make it
'                easy to either find if an error occured in the last Windows API
'                called, or display an error message for a specified error
'                number.
'
' Example Use  :
'
'   Private Declare Function SetForegroundWindow Lib "user32" (ByVal hwnd As Long) As Long
'
'   Private Sub Form_Load()
'   On Error Resume Next
'
'     SetForegroundWindow Me.hwnd
'     If GetLastError_Msg(, , "SetForegroundWindow") = True Then
'       Unload Me
'     End If
'
'   End Sub
'
'================================================================================

Public Const MAX_PATH = 260 + 1 ' Plus 1 to allow for NULL terminater
Public Const FORMAT_MESSAGE_FROM_SYSTEM = &H1000

Public Declare Sub SetLastError Lib "kernel32" (ByVal dwErrCode As Long)
Public Declare Function GetLastError Lib "kernel32" () As Long
Public Declare Function FormatMessage Lib "kernel32" Alias "FormatMessageA" (ByVal dwFlags As Long, lpSource As Any, ByVal dwMessageId As Long, ByVal dwLanguageId As Long, ByVal lpBuffer As String, ByVal nSize As Long, Arguments As Long) As Long

'================================================================================
' GetLastErr_Msg
'
' Purpose :
' Function that gets the last error caused by Windows API's.  This only works
' with functions that use the GetLastError function to return an error code.
' Not all Windows API's do.
'
' If no error has occured, no message is displayed.
'
' Param                 Use
' ------------------------------------
' ErrorNumber           Optional. Error number to display.  If this is set to
'                       zero, then the GetLastError API is called to see if an
'                       error occured.  If no error occured, the function exits.
' LastAPICalled         Optional. Tells the user which API caused the error.
'
' Return
' ------
' If no error occured, no message is displayed & function returns FALSE.
' If an error occured, an error message is displayed & the function returns TRUE.
'
'================================================================================
Public Function GetLastErr_Msg(Optional ByVal ErrorNumber As Long, Optional ByVal LastAPICalled As String = "last", Optional ByVal DisplayErrMsg As Boolean = True, Optional ByRef Return_ErrNum As Long, Optional ByRef Return_ErrMsg As String) As Boolean
On Error Resume Next
  
  Dim ErrMsg As String
  
  ' If no error message is specified then check for one
  If ErrorNumber = 0 Then
    ErrorNumber = GetLastError
    If ErrorNumber = 0 Then
      GetLastErr_Msg = False
      Exit Function
    End If
  End If
  
  ' Allocate a buffer for the error description
  ErrMsg = String(MAX_PATH, 0)
  
  ' Get the error description
  FormatMessage FORMAT_MESSAGE_FROM_SYSTEM, ByVal 0&, ErrorNumber, 0, ErrMsg, MAX_PATH, 0
  ErrMsg = Left(ErrMsg, InStr(ErrMsg, Chr(0)) - 1)
  
  ' Display the error message
  If DisplayErrMsg = True Then
    MsgBox "An error occured while calling the " & LastAPICalled & " Windows API function." & Chr(13) & "Below is the error information:" & Chr(13) & Chr(13) & "Error Number = " & CStr(ErrorNumber) & Chr(13) & "Error Description = " & ErrMsg, vbOKOnly + vbExclamation, "  Windows API Error"
  End If
  
  ' Set the returns
  Return_ErrNum = ErrorNumber
  Return_ErrMsg = ErrMsg
  GetLastErr_Msg = True
  
  ' Set the last error to 0 (no error) so next time through it doesn't report the same error twice
  SetLastError 0
  
End Function
